@@ -23,6 +23,9 @@
 #include <vector>
 #include <csignal>
 #include <cstring> // std::memset()
+#ifndef NDEBUG
+#include <iostream>
+#endif // NDEBUG
 
 extern "C"{
 #include <unistd.h> // ::getuid()
@@ -202,7 +205,7 @@ namespace
     } // TrapSignals()
 
 
-    const unsigned SiteSurveyTimerPeriod (250);
+    const unsigned SiteSurveyTimerPeriod (1000);
     const unsigned GlobalTimerPeriod (3000);
 
 } // anonymous namespace
@@ -236,7 +239,7 @@ bool nsGUI::AskUser (const std::string& Msg, ::GtkWindow* pWindow) throw()
 
 nsGUI::CGtkGUI::CGtkGUI () throw (nsErrors::CException)
     : m_pModule (0), m_pWindow (GTK_WINDOW (::gtk_window_new
-                                                    (GTK_WINDOW_TOPLEVEL))),
+                                                        (GTK_WINDOW_TOPLEVEL))),
       m_pNoteBook (GTK_NOTEBOOK (::gtk_notebook_new())), m_Exc ("", 0),
       m_IsRoot (IsReallyRoot()), m_GlobalTimer (0), m_SiteSurveyTab (this),
       m_SiteSurveyTabTimer (0), m_TimeRemaining (0), m_ProfileTab (this),
@@ -247,7 +250,8 @@ nsGUI::CGtkGUI::CGtkGUI () throw (nsErrors::CException)
 #else
       m_PrefsPage (m_IsRoot),
 #endif // NHELPER
-      m_IsIgnored (false), m_LeaveRootModeTimer (0)
+      m_IsIgnored (false), m_LeaveRootModeTimer (0), m_CurrentRateIndex (0),
+      m_StatsAvailable (true)
 {
     ::gtk_window_set_position (m_pWindow, GTK_WIN_POS_CENTER);
     ::gtk_window_set_resizable (m_pWindow, false);
@@ -282,13 +286,27 @@ nsGUI::CGtkGUI::CGtkGUI () throw (nsErrors::CException)
     TrapSignals();
     LoadOptions();
     SetWindowIcons();
+#ifndef NDEBUG
+    std::cerr << "GUI successfully initialized.\n";
+#endif // NDEBUG
 
 } // CGtkGUI()
 
 
-nsGUI::CGtkGUI::~CGtkGUI () throw()
+nsGUI::CGtkGUI::~CGtkGUI () throw (nsErrors::CException)
 {
-    StoreOptions();
+    try
+    {
+        if (m_pModule.get())
+            StoreData();
+    }
+    catch (const nsErrors::CException&)
+    {
+        ::gtk_widget_destroy (GTK_WIDGET (m_pWindow));
+        if (m_GlobalTimer) ::g_source_remove (m_GlobalTimer);
+        if (m_SiteSurveyTabTimer) ::g_source_remove (m_SiteSurveyTabTimer);
+        throw;
+    }
     ::gtk_widget_destroy (GTK_WIDGET (m_pWindow));
     if (m_GlobalTimer) ::g_source_remove (m_GlobalTimer);
     if (m_SiteSurveyTabTimer) ::g_source_remove (m_SiteSurveyTabTimer);
@@ -323,22 +341,80 @@ void nsGUI::CGtkGUI::SetWindowIcons () throw()
 } // SetWindowIcons()
 
 
-void nsGUI::CGtkGUI::Run () throw (nsErrors::CException)
+void nsGUI::CGtkGUI::Run (const nsCore::CCLIOpt& Opt)
+                                                    throw (nsErrors::CException)
 {
-    try{if (!ChangeModule()) return;}
-    catch (const nsErrors::CException& Exc) {if (ProcessError (Exc)) throw;}
+#ifndef NDEBUG
+    std::cerr << "Loading module...\n";
+#endif // NDEBUG
+    try{if (!ChangeModule (Opt.IfaceName())) return;}
+    catch (const nsErrors::CException& Exc)
+    {
+#ifndef NDEBUG
+        std::cerr << "Couldn't load module : " << Exc.GetMsg() << ' '
+                  << Exc.GetCode() << std::endl;
+#endif // NDEBUG
+        if (Exc.GetCode() == m_Exc.GetCode() || ProcessError (Exc) ||
+                                                            !m_pModule.get())
+            throw;
+    }
         // We can't set those signals before having a module.
     ::g_signal_connect (G_OBJECT (m_pWindow), "delete_event",
                         G_CALLBACK (DeleteCallBack), 0);
     ::g_signal_connect (G_OBJECT (m_pNoteBook), "switch-page",
                         G_CALLBACK (PageSwitchCallBack), this);
 
-    if (!m_PrefsPage.StartMinimized())
+    if (!m_PrefsPage.StartMinimized() && !Opt.Hidden())
         ::gtk_widget_show_all (GTK_WIDGET (m_pWindow));
     if (m_PrefsPage.DisplayTrayIcon())
         m_TrayIcon.Show();
+
+    if (Opt.Profile())
+    {
+        unsigned I (0);
+        for ( ; I < m_pModule->GetNbProfile() ; ++I)
+            if (m_pModule->GetProfile (I).GetName() == Opt.ProfileName())
+            {
+                if (BecomeRoot())
+                {
+#ifndef NDEBUG
+                    std::cerr << "Applying the profile (command line)...\n";
+#endif // NDEBUG
+                    m_pModule->ApplyProfile (I);
+                }
+#ifndef NDEBUG
+                else
+                    std::cerr << "Cannot apply the profile (command line) :"
+                                 " not root.\n";
+#endif // NDEBUG
+                break;
+            }
+        if (I == m_pModule->GetNbProfile())
+            throw (nsErrors::CException (std::string ("Profile not found : ") +
+                                 Opt.ProfileName(), nsErrors::ProfileNotFound));
+    }
+    if (Opt.Dhcp() && BecomeRoot())
+    {
+#ifndef NDEBUG
+        std::cerr << "Running DHCP client (command line option).\n";
+#endif // NDEBUG
+        m_pIface->SetIPFromDHCP (m_pModule->GetDriver().GetSSID());
+    }
+
+#ifndef NDEBUG
+    std::cerr << "Running main loop...\n";
+#endif // NDEBUG
     ::gtk_main();
-    if (m_Exc.GetCode()) throw nsErrors::CException (m_Exc);
+#ifndef NDEBUG
+    std::cerr << "Exiting main loop...\n";
+#endif // NDEBUG
+    if (m_Exc.GetCode())
+    {
+#ifndef NDEBUG
+        std::cerr << "Error.\n";
+#endif // NDEBUG
+        throw nsErrors::CException (m_Exc);
+    }
 
 } // Run()
 
@@ -358,7 +434,7 @@ void nsGUI::CGtkGUI::PageSwitchCallBack (::GtkNotebook* Notebook,
           break;
 
           case Stats :
-            This->RefreshStats();
+            This->m_StatsAvailable = This->RefreshStats();
           break;
 
           case SiteSurvey :
@@ -394,9 +470,18 @@ bool nsGUI::CGtkGUI::GlobalTimerWithTrayCallBack (CGtkGUI* This) throw()
                         This->m_pModule->GetDriver().GetProtoName() + " (" +
                     This->m_pModule->GetDriver().GetDeviceName() + ")\n\n" +
                         "ESSID : " + This->m_pModule->GetDriver().GetSSID();
-            const nsWireless::CQuality Quality
-                                (This->m_pModule->GetDriver().GetQuality());
-            const unsigned TxRate (This->m_pModule->GetDriver().GetTxRate());
+            nsWireless::CQuality Quality;
+            unsigned TxRate (0);
+            try
+            {
+                Quality = This->m_pModule->GetDriver().GetQuality();
+            }
+            catch (...) {}
+            try
+            {
+                TxRate = This->m_pModule->GetDriver().GetTxRate();
+            }
+            catch (...) {}
             This->RefreshTray (Quality, TxRate);
             if (GTK_WIDGET_VISIBLE (GTK_WIDGET (This->m_pWindow)))
                 switch (::gtk_notebook_get_current_page (This->m_pNoteBook))
@@ -406,7 +491,8 @@ bool nsGUI::CGtkGUI::GlobalTimerWithTrayCallBack (CGtkGUI* This) throw()
                   break;
 
                   case Stats :
-                    This->RefreshStats();
+                    if (This->m_StatsAvailable)
+                        This->m_StatsAvailable = This->RefreshStats();
                   break;
 
                   case SiteSurvey :
@@ -438,7 +524,8 @@ bool nsGUI::CGtkGUI::GlobalTimerWithoutTrayCallBack (CGtkGUI* This) throw()
               break;
 
               case Stats :
-                This->RefreshStats();
+                if (This->m_StatsAvailable)
+                    This->m_StatsAvailable = This->RefreshStats();
               break;
 
               case SiteSurvey :
@@ -476,6 +563,21 @@ bool nsGUI::CGtkGUI::Connect () throw()
                                     [m_SiteSurveyTab.GetSelectedRow()]);
             if (FillCell (Cell))
             {
+                /* FIXME The interface is not brought down, this is require
+                         however when changing the mode with rt2x00.
+                if (m_pIface->IsUp())
+                {
+                    m_pIface->Down();
+                    try{m_pModule->Connect (Cell);}
+                    catch (...)
+                    {
+                        m_pIface->Up();
+                        throw;
+                    }
+                    m_pIface->Up();
+                }
+                else
+                */
                 m_pModule->Connect (Cell);
                 SelectMatchingProfile (Cell);
                     // TODO Use FillCell() instead of AskUser() ?
@@ -510,16 +612,15 @@ bool nsGUI::CGtkGUI::FillCell (nsWireless::CCell& Cell) throw()
         return true;
     ::GtkDialog* const pDialog (GTK_DIALOG (::gtk_dialog_new_with_buttons
                                             ("Connection", m_pWindow,
-                                             ::GtkDialogFlags(
-                                                     GTK_DIALOG_MODAL |
-                                                     GTK_DIALOG_NO_SEPARATOR),
+                                             ::GtkDialogFlags
+                                                            (GTK_DIALOG_MODAL),
                                              GTK_STOCK_CANCEL,
                                              GTK_RESPONSE_CANCEL,
                                              GTK_STOCK_OK, GTK_RESPONSE_OK,
                                              reinterpret_cast<void*> (0))));
     ::gtk_window_set_resizable (GTK_WINDOW (pDialog), false);
     ::GtkWidget* pSSIDEntry (0);
-    CKeyEntry KeyEntry;
+    CMultiKey<4> KeyEntries;
     const bool IsEncrypted (Cell.GetEncryptionD().GetEncrypt() !=
                                                             nsWireless::None);
     if (Cell.GetSSID().empty())
@@ -529,28 +630,28 @@ bool nsGUI::CGtkGUI::FillCell (nsWireless::CCell& Cell) throw()
         pSSIDEntry = ::gtk_entry_new();
         ::gtk_entry_set_max_length (GTK_ENTRY (pSSIDEntry),
                                     m_pModule->GetDriver().GetMaxSSIDSize());
-        if (IsEncrypted)
+        if (!IsEncrypted)
             ::g_signal_connect (pSSIDEntry, "activate",
-                                G_CALLBACK (SetFocusCallBack),
-                                KeyEntry.operator ::GtkEntry*());
-         else ::g_signal_connect (pSSIDEntry, "activate",
                                 G_CALLBACK (TriggerResponseCallBack), pDialog);
         ::gtk_box_pack_start_defaults (GTK_BOX (pDialog->vbox), pSSIDEntry);
     }
 
     if (IsEncrypted)
     {
-        ::GtkWidget* const pHBox (::gtk_hbox_new (false, 6));
-        ::gtk_box_pack_start_defaults (GTK_BOX (pHBox),
-                                       ::gtk_image_new_from_stock
-                                            (GTK_STOCK_DIALOG_AUTHENTICATION,
-                                             GTK_ICON_SIZE_SMALL_TOOLBAR));
-        ::gtk_box_pack_start_defaults (GTK_BOX (pHBox),
-                                       ::gtk_label_new ("Enter the key : "));
-        ::g_signal_connect (KeyEntry.operator ::GtkEntry*(), "activate",
-                            G_CALLBACK (TriggerResponseCallBack), pDialog);
-        ::gtk_box_pack_start_defaults (GTK_BOX (pDialog->vbox), pHBox);
-        ::gtk_box_pack_start_defaults (GTK_BOX (pDialog->vbox), KeyEntry);
+        if (Cell.GetEncryptionD().GetEncrypt() != nsWireless::WEP)
+        {
+            KeyEntries.GetKeyEntry (0).ASCIIOnly (true);
+            KeyEntries.AdditionnalKeyAvailable (false);
+        }
+        else
+            ::g_signal_connect (KeyEntries.GetKeyEntry
+                            (KeyEntries.GetNbKey() - 1).operator GtkEntry*(),
+                                "activate",
+                                G_CALLBACK (TriggerResponseCallBack), pDialog);
+        ::g_signal_connect (KeyEntries.GetKeyEntry (0).operator GtkEntry*(),
+                            "activate", G_CALLBACK (TriggerResponseCallBack),
+                            pDialog);
+        ::gtk_box_pack_start_defaults (GTK_BOX (pDialog->vbox), KeyEntries);
     }
 
     ::gtk_box_set_spacing (GTK_BOX (pDialog->vbox), 12);
@@ -570,7 +671,11 @@ bool nsGUI::CGtkGUI::FillCell (nsWireless::CCell& Cell) throw()
             }
         }
         if (IsEncrypted)
-            Cell.GetEncryptionD().SetKey (KeyEntry.Get());
+        {
+            for (unsigned I (0) ; I < 4 ; ++I)
+                Cell.GetEncryptionD().SetKey (KeyEntries.Get (I), I);
+            Cell.GetEncryptionD().SetDefaultKey (KeyEntries.GetDefaultKey());
+        }
         IsOk = true;
         break;
     }
@@ -590,19 +695,22 @@ void nsGUI::CGtkGUI::Scan () throw()
             m_CellVec.clear();
             m_pModule->GetDriver().Scan();
             m_SiteSurveyTab.ScanningStarted();
-            m_TimeRemaining = SiteSurveyTimerPeriod * 60;
+            m_TimeRemaining = 15000; // The driver has 15 seconds.
             m_SiteSurveyTabTimer = ::g_timeout_add (SiteSurveyTimerPeriod,
                              reinterpret_cast< ::gboolean (*) (void*)>
                                             (RefreshCellListCallBack), this);
         }
     }
-    catch (const nsErrors::CSystemExc& Exc) {ProcessError (Exc);}
+    catch (const nsErrors::CException& Exc) {ProcessError (Exc);}
 
 } // Scan()
 
 
 bool nsGUI::CGtkGUI::RefreshCellListCallBack (CGtkGUI* This) throw()
 {
+#ifndef NDEBUG
+    std::cerr << "CGtkGUI::RefreshCellListCallBack()\n";
+#endif // NDEBUG
     This->m_TimeRemaining -= SiteSurveyTimerPeriod;
     try
     {
@@ -610,25 +718,42 @@ bool nsGUI::CGtkGUI::RefreshCellListCallBack (CGtkGUI* This) throw()
             throw nsErrors::CException ("Failed to get scan result in time.",
                                         nsErrors::ScanningTimeOut);
         This->m_pModule->GetDriver().GetScanResult (This->m_CellVec);
+#ifndef NDEBUG
+        std::cerr << "\tGot the results.\n";
+#endif // NDEBUG
         const nsWireless::CMacAddress Addr
                                 (This->m_pModule->GetDriver().GetAPMacAddr());
         const nsWireless::CCell* const pCurrentCell
                     (This->m_SiteSurveyTab.AddCells (This->m_CellVec, Addr));
-        if (pCurrentCell) This->SelectMatchingProfile (*pCurrentCell);
+        if (pCurrentCell)
+        {
+#ifndef NDEBUG
+            std::cerr << "\tSelecting matching profile.\n";
+#endif // NDEBUG
+            This->SelectMatchingProfile (*pCurrentCell);
+        }
     }
     catch (const std::bad_alloc& Exc)
     {
         This->ProcessError (nsErrors::CException (Exc.what(),
                             nsErrors::OutOfMemory));
+        return false;
     }
     catch (const nsErrors::CException& Exc)
     {
+#ifndef NDEBUG
+        std::cerr << "\tError : " << Exc.GetMsg() << ' ' << Exc.GetCode()
+                  << std::endl;
+#endif // NDEBUG
         if (Exc.GetCode() == EINTR || Exc.GetCode() == EAGAIN)
             return true; // We go for another SiteSurveyTimerPeriod.
         This->ProcessError (Exc);
     }
     This->m_SiteSurveyTab.ScanningStopped();
     This->m_SiteSurveyTabTimer = 0;
+#ifndef NDEBUG
+    std::cerr << "Exiting CGtkGUI::RefreshCellListCallBack()\n";
+#endif // NDEBUG
     return false;
 
 } // RefreshCellListCallBack()
@@ -638,23 +763,33 @@ bool nsGUI::CGtkGUI::ProcessError (const nsErrors::CException& Exc) throw()
 {
     switch (Exc.GetCode())
     {
+      case nsErrors::ScanningTimeOut :
       case nsErrors::RT2400AdHocLimitation :
+      case nsErrors::CannotExecuteHelper :
         NotifyError (Exc.GetMsg(), 0, m_pWindow);
       return false;
 
       case EPIPE :
         NotifyError ("Your privileged session has been terminated, please try "
-                     "again.", EPIPE, m_pWindow);
+                     "again.", 0, m_pWindow);
         SetNoRoot();
       return false;
 
+      case EOPNOTSUPP :
+        NotifyError (Exc.GetMsg() + "\nOperation not supported by the driver.",
+                     0, m_pWindow);
+      break;
+
+      case ENETDOWN :
+        NotifyError ("Failed : network interface is down.", 0, m_pWindow);
+      return false;
+
       case EPERM :
         SetNoRoot();
       // No break.
 
-      case nsErrors::NoMatchingFreq : case EAGAIN : case EBUSY :
-      case EINVAL : case ENOSYS : case ENOTCONN : case nsErrors::InvalidData :
-      case nsErrors::ScanningTimeOut : case nsErrors::InvalidRootPassword :
+      case EAGAIN : case EBUSY : case EINVAL : case ENOSYS : case ENOTCONN :
+      case nsErrors::InvalidData : case nsErrors::InvalidRootPassword :
       case ENOENT :
           NotifyError (std::string ("An error occured :\n") + Exc.GetMsg(),
                        Exc.GetCode(), m_pWindow);
@@ -721,22 +856,39 @@ void nsGUI::CGtkGUI::SelectMatchingProfile (const nsWireless::CCell& Cell)
 } // SelectMatchingProfile()
 
 
-bool nsGUI::CGtkGUI::ChangeModule () throw (nsErrors::CException)
+bool nsGUI::CGtkGUI::ChangeModule (const std::string& IfaceName)
+                                                    throw (nsErrors::CException)
 {
+#ifndef NDEBUG
+    std::cerr << "CGtkGUI::ChangeModule()\n";
+#endif // NDEBUG
     std::vector<nsWireless::CDeviceInfo> DevicesVec;
     nsCore::GetWirelessDevice (DevicesVec);
     if (DevicesVec.empty())
         throw nsErrors::CException ("Can't find any wireless network"
-                                    " interface.\nDo you have the wireless"
-                                    " extensions compiled in your kernel?",
-                                    nsErrors::NoDeviceFound);
-    const int Pos (DevicesVec.size() == 1 && !m_pModule.get() ? 0
-                                            : SelectInterface (DevicesVec));
+                                    " interface.", nsErrors::NoDeviceFound);
+    int Pos (0);
+    if (IfaceName.empty())
+    {
+        if (DevicesVec.size() > 1 || m_pModule.get())
+            Pos = SelectInterface (DevicesVec);
+    }
+    else
+    {
+        for ( ; unsigned (Pos) < DevicesVec.size() &&
+                        DevicesVec [Pos].GetDeviceName() != IfaceName ; ++Pos);
+        if (unsigned (Pos) == DevicesVec.size())
+            throw nsErrors::CException (std::string ("Can't find interface : ")
+                                        + IfaceName, nsErrors::NoDeviceFound);
+    }
     if (Pos == -1 || m_pModule.get() &&
                                     m_pModule->GetDriver().GetDeviceName() ==
                                             DevicesVec [Pos].GetDeviceName())
         return false;
     nsCore::IModule* pModule (0);
+#ifndef NDEBUG
+    std::cerr << "Instanciating module...\n";
+#endif // NDEBUG
     try{pModule = nsCore::MakeModule (DevicesVec [Pos], this);}
     catch (const std::bad_alloc& Exc)
     {
@@ -751,6 +903,9 @@ bool nsGUI::CGtkGUI::ChangeModule () throw (nsErrors::CException)
     {
         if (m_IsRoot && !IsReallyRoot())
             m_pModule->BecomeRoot();
+#ifndef NDEBUG
+        std::cerr << "Instanciating interface...\n";
+#endif // NDEBUG
         m_pIface.reset (new nsNet::CInterface
                                     (m_pModule->GetDriver().GetDeviceName()));
     }
@@ -774,6 +929,13 @@ bool nsGUI::CGtkGUI::ChangeModule () throw (nsErrors::CException)
     }
     m_CellVec.clear();
     RefreshCellListCallBack (this);
+    if (m_Exc.GetCode())
+    {
+#ifndef NDEBUG
+        std::cerr << "Error detected after fetching scan results.\n";
+#endif // NDEBUG
+        throw m_Exc;
+    }
     RefreshRates();
     if (m_PrefsPage.DisplayTrayIcon())
     {
@@ -789,6 +951,9 @@ bool nsGUI::CGtkGUI::ChangeModule () throw (nsErrors::CException)
                                     (GlobalTimerWithoutTrayCallBack), this);
         GlobalTimerWithoutTrayCallBack (this);
     }
+#ifndef NDEBUG
+    std::cerr << "Exiting CGtkGUI::ChangeModule()\n";
+#endif // NDEBUG
     return true;
 
 } // ChangeModule()
@@ -801,11 +966,7 @@ const void nsGUI::CGtkGUI::NewProfile (unsigned Pos) throw()
     try
     {
         if (m_pModule->GetDriver().GetAPMacAddr() == Cell.GetAPMacAddress())
-        {
             IsCurrentCell = true;
-            if (m_IsRoot)
-                Cell.GetEncryptionD() = m_pModule->GetDriver().GetEncryption();
-        }
     }
     catch (const nsErrors::CSystemExc& Exc) {} // Ignored.
     const nsUserData::CProfile* const pProfile (m_pModule->NewProfile (Cell));
@@ -842,7 +1003,7 @@ void nsGUI::CGtkGUI::SelectCurrentCell (const nsWireless::CMacAddress& Addr)
 } // SelectCurrentCell()
 
 
-bool nsGUI::CGtkGUI::BecomeRoot () throw (nsErrors::CSystemExc)
+bool nsGUI::CGtkGUI::BecomeRoot () throw (nsErrors::CException)
 {
     if (!m_IsRoot)
     {
@@ -869,7 +1030,10 @@ bool nsGUI::CGtkGUI::BecomeRoot () throw (nsErrors::CSystemExc)
             AutoLeaveRootModeCallBack (this);
         }
         else
+        {
             NotifyError ("An error occured.\nPlease try again.", 0, m_pWindow);
+            SetNoRoot();
+        }
 #else 
         ::GtkDialog* const pDialog (GTK_DIALOG (::gtk_dialog_new_with_buttons
                             ("Enter password", m_pWindow,
@@ -906,15 +1070,18 @@ bool nsGUI::CGtkGUI::BecomeRoot () throw (nsErrors::CSystemExc)
                     m_IsRoot = true;
                     AutoLeaveRootModeCallBack (this);
                 }
-                else if (!IsMsgDisplayed)
+                else
                 {
-                    ::GtkWidget* const pLabel (::gtk_label_new
-                                ("Invalid password, please try again."));
-                    ::gtk_box_pack_start_defaults (GTK_BOX (pDialog->vbox),
-                                                   pLabel);
-                    ::gtk_widget_show (pLabel);
                     ::gtk_entry_set_text (pEntry, "");
-                    IsMsgDisplayed = true;
+                    if (!IsMsgDisplayed)
+                    {
+                        ::GtkWidget* const pLabel (::gtk_label_new
+                                    ("Invalid password, please try again."));
+                        ::gtk_box_pack_start_defaults (GTK_BOX (pDialog->vbox),
+                                                       pLabel);
+                        ::gtk_widget_show (pLabel);
+                        IsMsgDisplayed = true;
+                    }
                 }
         }
         catch (const std::bad_alloc& Exc)
@@ -925,7 +1092,7 @@ bool nsGUI::CGtkGUI::BecomeRoot () throw (nsErrors::CSystemExc)
             m_IsIgnored = false;
             throw nsErrors::CSystemExc (Exc.what(), nsErrors::OutOfMemory);
         }
-        catch (const nsErrors::CSystemExc& Exc)
+        catch (const nsErrors::CException& Exc)
         {
             ::gtk_widget_destroy (GTK_WIDGET (pDialog));
             m_IsIgnored = true;
@@ -982,16 +1149,25 @@ void nsGUI::CGtkGUI::SetIPFromProfile () throw (nsErrors::CException)
 
 void nsGUI::CGtkGUI::RefreshLinkStatus () throw (nsErrors::CException)
 {
-    RefreshLinkStatus (m_pModule->GetDriver().GetQuality(),
-                       m_pModule->GetDriver().GetTxRate(),
-                       m_pModule->GetDriver().GetAPMacAddr());
+    nsWireless::CQuality Quality;
+    unsigned TxRate (0);
+    try
+    {
+        Quality = m_pModule->GetDriver().GetQuality();
+    }
+    catch (...) {}
+    try
+    {
+        TxRate = m_pModule->GetDriver().GetTxRate();
+    }
+    catch (...) {}
+    RefreshLinkStatus (Quality, TxRate, m_pModule->GetDriver().GetAPMacAddr());
 
 } // RefreshLinkStatus()
 
 
 void nsGUI::CGtkGUI::RefreshLinkStatus (const nsWireless::CQuality& Quality,
-                                        unsigned TxRate,
-                                    const nsWireless::CMacAddress& APMacAddr)
+                    unsigned TxRate, const nsWireless::CMacAddress& APMacAddr)
                                                 throw (nsErrors::CException)
 {
     m_LinkStatusTab.SetChannel (m_pModule->GetDriver().GetChannel(),
@@ -1060,7 +1236,11 @@ void nsGUI::CGtkGUI::RefreshTray (const nsWireless::CQuality& Quality,
        << Quality.GetLinkQuality() << " %\n"
        << "Signal level : " << Quality.GetSignalLevel() << " dBm\n"
        << "Noise level : " << Quality.GetNoiseLevel() << " dBm\n"
-          "Tx rate : " << TxRate / 1000.0 << " Mbps";
+          "Tx rate : ";
+    if (TxRate)
+        Os << TxRate / 1000.0 << " Mbps";
+    else
+        Os << "Unknown.";
     m_TrayIcon.SetTooltip (Os.str());
 
 } // RefreshTray()
@@ -1078,7 +1258,7 @@ void nsGUI::CGtkGUI::RootModeChangedCallBack (CGtkGUI* This) throw()
         }
         catch (const nsErrors::CException& Exc)
         {
-            if (!This->ProcessError (Exc)) {This->SetNoRoot();}
+            if (!This->ProcessError (Exc)) This->SetNoRoot();
         }
     }
 
@@ -1167,25 +1347,43 @@ void nsGUI::CGtkGUI::SetNoRoot () throw()
 
 void nsGUI::CGtkGUI::RefreshRates () throw()
 {
+#ifndef NDEBUG
+    std::cerr << "CGtkGUI::RefreshRates()\n";
+#endif // NDEBUG
     std::vector<int> RatesVec;
     try {m_pModule->GetDriver().GetSupportedRates (RatesVec);}
-    catch (...) {} // Ignore everything.
+    catch (const nsErrors::CException& Exc)
+    {
+#ifndef NDEBUG
+        std::cerr << "Cannot get supported rates : " << Exc.GetMsg() << ' '
+                  << Exc.GetCode() << std::endl;
+#endif // NDEBUG
+    }
     m_PrefsPage.SetRates (RatesVec);
-    unsigned Index (0);
     if (!RatesVec.empty())
+    {
+        unsigned Index (0);
         try
         {
-            if (m_IsRoot) // Restore to the maximum rate.
-                m_pModule->GetDriver().SetTxRate (RatesVec.front());
             Index = std::find (RatesVec.begin(), RatesVec.end(),
                                m_pModule->GetDriver().GetTxRate()) -
                                                             RatesVec.begin();
         }
-        catch (...) {} // Ignore everything.
-    m_IsIgnored = true;
-    m_PrefsPage.SetRateIndex (Index);
-    m_CurrentRateIndex = Index;
-    m_IsIgnored = false;
+        catch (const nsErrors::CException& Exc)
+        {
+#ifndef NDEBUG
+            std::cerr << "Cannot get current TX rate : " << Exc.GetMsg() << ' '
+                      << Exc.GetCode() << std::endl;
+#endif // NDEBUG
+        }
+        m_IsIgnored = true;
+        m_PrefsPage.SetRateIndex (Index);
+        m_CurrentRateIndex = Index;
+        m_IsIgnored = false;
+    }
+#ifndef NDEBUG
+    std::cerr << "Exiting CGtkGUI::RefreshRates()\n";
+#endif // NDEBUG
 
 } // RefreshRates()
 
@@ -1240,7 +1438,7 @@ void nsGUI::CGtkGUI::LoadOptions () throw()
 } // LoadOptions()
 
 
-void nsGUI::CGtkGUI::StoreOptions () throw()
+void nsGUI::CGtkGUI::StoreData () throw (nsErrors::CException)
 {
     try
     {
@@ -1248,12 +1446,13 @@ void nsGUI::CGtkGUI::StoreOptions () throw()
                                               m_PrefsPage.StartMinimized(),
                                           m_PrefsPage.GetAutoLeaveRootDelay());
         OptionsFile.Record();
+        m_pModule->RecordProfiles();
     }
     catch (const nsErrors::CException& Exc)
     {
-        NotifyError ("The options could not have been recorded : " +
-                                                                Exc.GetMsg(),
+        NotifyError ("Options and profiles could not have been recorded.",
                      Exc.GetCode(), m_pWindow);
+        throw;
     }
 
-} // StoreOptions()
+} // StoreData()
