@@ -45,12 +45,12 @@ namespace
     const nsWireless::AuthType_e TabAuthRT2400 [TabAuthRT2400Size] =
                                         {nsWireless::Open, nsWireless::Shared};
 
-    const unsigned TabAuthRT25x0Size (3);
-    const nsWireless::AuthType_e TabAuthRT25x0 [TabAuthRT25x0Size] =
+    const unsigned TabAuthRT2500Size (3);
+    const nsWireless::AuthType_e TabAuthRT2500 [TabAuthRT2500Size] =
                     {nsWireless::Open, nsWireless::Shared, nsWireless::WPAPSK};
 
-    const unsigned TabAuthAdHocRT25x0Size (3);
-    const nsWireless::AuthType_e TabAuthAdHocRT25x0 [TabAuthAdHocRT25x0Size] =
+    const unsigned TabAuthAdHocRTxxSize (3);
+    const nsWireless::AuthType_e TabAuthAdHocRTxx [TabAuthAdHocRTxxSize] =
                 {nsWireless::Open, nsWireless::Shared, nsWireless::WPANONE};
 
     const unsigned TabNoneEncryptSize (2);
@@ -61,19 +61,44 @@ namespace
     const nsWireless::EncryptType_e TabSharedEncrypt [TabSharedEncryptSize] =
                                                             {nsWireless::WEP};
 
-    const unsigned TabWPAPSKRT2500EncryptSize (2);
-    const nsWireless::EncryptType_e TabWPAPSKRT2500Encrypt
-            [TabWPAPSKRT2500EncryptSize] = {nsWireless::TKIP, nsWireless::AES};
+    const unsigned TabWPAPSKEncryptSize (2);
+    const nsWireless::EncryptType_e TabWPAPSKEncrypt [TabWPAPSKEncryptSize] =
+                                            {nsWireless::TKIP, nsWireless::AES};
 
-    const unsigned TabWPAPSKRT2570EncryptSize (1);
-    const nsWireless::EncryptType_e TabWPAPSKRT2570Encrypt
-                            [TabWPAPSKRT2570EncryptSize] = {nsWireless::TKIP};
+    const unsigned TabAuthRTxxSize (4);
+    const nsWireless::AuthType_e TabAuthRTxx [TabAuthRTxxSize] =
+                                    {nsWireless::Open, nsWireless::Shared,
+                                     nsWireless::WPAPSK, nsWireless::WPA2PSK};
 
-        // FIXME Are they all really supported by rt2400 ?
-    const unsigned TabModeRT2xx0Size (3);
-    const nsWireless::Mode_e TabModeRT2xx0 [TabModeRT2xx0Size] =
+    const unsigned TabModeRTxxSize (3);
+    const nsWireless::Mode_e TabModeRTxx [TabModeRTxxSize] =
                 {nsWireless::Managed, nsWireless::AdHoc, nsWireless::Monitor};
 
+
+    void ApplyRT2400Profile (const nsUserData::CProfile& Profile,
+                             nsWireless::CRT2400Driver& Driver)
+                                                    throw (nsErrors::CSystemExc)
+    {
+        Driver.SetChannel (Profile.GetChannel());
+        Driver.SetMode (Profile.GetMode());
+        Driver.SetEncryption (Profile.GetEncryptionD());
+        Driver.SetSSID (Profile.GetSSID());  
+
+    } // ApplyRT2400Profile()
+
+
+    void ApplyRTxxProfile (const nsUserData::CProfile& Profile,
+                             nsWireless::IDriver& Driver)
+                                                    throw (nsErrors::CSystemExc)
+    {
+        Driver.SetMode (Profile.GetMode());
+        Driver.SetChannel (Profile.GetChannel());
+        Driver.SetSSID (Profile.GetSSID());  
+        Driver.SetEncryption (Profile.GetEncryptionD());
+        Driver.SetSSID (Profile.GetSSID());  
+
+    } // ApplyRTxxProfile()
+
 } // anonymous namespace
 
 
@@ -81,42 +106,79 @@ namespace
 const char* const nsCore::CRT2400Module::NewProfileDefaultName ("New profile");
 
 
-const char* const nsCore::CRT2500Module::NewProfileDefaultName ("New profile");
+const char* const nsCore::CRTAdvancedModule::NewProfileDefaultName
+                                                                ("New profile");
 
 
 const char* const nsCore::CRT2570Module::NewProfileDefaultName ("New profile");
 
 
 
-nsGUI::CRT25x0ConfigDialog::CRT25x0ConfigDialog
-                            (const std::vector<nsWireless::CFreq>& FreqVec,
-                             const nsWireless::EncryptType_e* TabWPAEncrypt,
-                             unsigned TabWPAEncryptSize) throw()
-    : CSharedConfigDialog (TabModeRT2xx0, TabModeRT2xx0Size, FreqVec,
-                           TabAuthRT25x0, TabAuthRT25x0Size,
+void nsCore::CRT2400CLIModule::ApplyProfile (unsigned Pos)
+                                                    throw (nsErrors::CSystemExc)
+{
+    ApplyRT2400Profile (m_ProfilesFile [Pos], m_Driver);
+
+} // ApplyProfile()
+
+
+nsGUI::CRT2500ConfigDialog::CRT2500ConfigDialog () throw()
+    : CSharedConfigDialog (TabModeRTxx, TabModeRTxxSize,
+                           TabAuthRT2500, TabAuthRT2500Size,
                            TabNoneEncrypt, TabNoneEncryptSize,
                            TabSharedEncrypt, TabSharedEncryptSize,
-                           TabWPAEncrypt, TabWPAEncryptSize)
+                           TabWPAPSKEncrypt, TabWPAPSKEncryptSize)
 {
     ::g_signal_connect (G_OBJECT (GetModeBox().operator ::GtkWidget*()),
                         "changed", G_CALLBACK (ModeChangedCallBack), this);
 
-} // CRT25x0ConfigDialog
+} // CRT2500ConfigDialog
 
 
-void nsGUI::CRT25x0ConfigDialog::ModeChangedCallBack (::GtkToggleButton*,
-                                          CRT25x0ConfigDialog* This) throw()
+void nsGUI::CRT2500ConfigDialog::ModeChangedCallBack (::GtkToggleButton*,
+                                          CRT2500ConfigDialog* This) throw()
 {
     if (This->GetModeBox().GetActive() == nsWireless::AdHoc)
     {
-        This->GetAuthBox().SetElem (TabAuthAdHocRT25x0,
-                                    TabAuthAdHocRT25x0Size);
-        This->GetAuthBox().SetActive (TabAuthAdHocRT25x0 [0]);
+        This->GetAuthBox().SetElem (TabAuthAdHocRTxx,
+                                    TabAuthAdHocRTxxSize);
+        This->GetAuthBox().SetActive (TabAuthAdHocRTxx [0]);
     }
     else
     {
-        This->GetAuthBox().SetElem (TabAuthRT25x0, TabAuthRT25x0Size);
-        This->GetAuthBox().SetActive (TabAuthRT25x0 [0]);
+        This->GetAuthBox().SetElem (TabAuthRT2500, TabAuthRT2500Size);
+        This->GetAuthBox().SetActive (TabAuthRT2500 [0]);
+    }
+
+} // ModeChangedCallBack()
+
+
+nsGUI::CRTxxConfigDialog::CRTxxConfigDialog () throw()
+    : CSharedConfigDialog (TabModeRTxx, TabModeRTxxSize,
+                           TabAuthRTxx, TabAuthRTxxSize,
+                           TabNoneEncrypt, TabNoneEncryptSize,
+                           TabSharedEncrypt, TabSharedEncryptSize,
+                           TabWPAPSKEncrypt, TabWPAPSKEncryptSize)
+{
+    ::g_signal_connect (G_OBJECT (GetModeBox().operator ::GtkWidget*()),
+                        "changed", G_CALLBACK (ModeChangedCallBack), this);
+
+} // CRTxxConfigDialog
+
+
+void nsGUI::CRTxxConfigDialog::ModeChangedCallBack (::GtkToggleButton*,
+                                              CRTxxConfigDialog* This) throw()
+{
+    if (This->GetModeBox().GetActive() == nsWireless::AdHoc)
+    {
+        This->GetAuthBox().SetElem (TabAuthAdHocRTxx,
+                                    TabAuthAdHocRTxxSize);
+        This->GetAuthBox().SetActive (TabAuthAdHocRTxx [0]);
+    }
+    else
+    {
+        This->GetAuthBox().SetElem (TabAuthRTxx, TabAuthRTxxSize);
+        This->GetAuthBox().SetActive (TabAuthRTxx [0]);
     }
 
 } // ModeChangedCallBack()
@@ -127,7 +189,7 @@ nsCore::CRT2400Module::CRT2400Module (const std::string& DeviceName,
                                  throw (nsErrors::CException, std::bad_alloc)
     : m_pDriver (new nsWireless::CRT2400Driver (DeviceName)),
       m_ProfileEditor (new nsGUI::CSharedConfigDialog
-                                      (TabModeRT2xx0, TabModeRT2xx0Size,
+                                      (TabModeRTxx, TabModeRTxxSize,
                                        m_pDriver->GetSupportedFreq(),
                                        TabAuthRT2400, TabAuthRT2400Size,
                                        TabNoneEncrypt, TabNoneEncryptSize,
@@ -140,22 +202,22 @@ nsCore::CRT2400Module::CRT2400Module (const std::string& DeviceName,
 } // CRT2400Module()
 
 
-void nsCore::CRT2400Module::BecomeRoot () throw (nsErrors::CSystemExc,
+void nsCore::CRT2400Module::BecomeRoot () throw (nsErrors::CException,
                                                  std::bad_alloc)
 {
-    try
-    {
-        m_pDriver.reset (new nsWireless::CRT2400DriverRoot
+    m_pDriver.reset (new nsWireless::CRT2400DriverRoot
                                                 (m_pDriver->GetDeviceName()));
-    }
-    catch (const nsErrors::CException& Exc)
-    {
-        throw nsErrors::CSystemExc (Exc.GetMsg(), Exc.GetCode());
-    }
 
 } // BecomeRoot()
 
 
+void nsCore::CRT2400Module::RecordProfiles () throw (nsErrors::CException)
+{
+    m_ProfilesFile.Record();
+
+} // RecordProfiles()
+
+
 void nsCore::CRT2400Module::Connect (const nsWireless::CCell& Cell)
                                    throw (nsErrors::CSystemExc, std::bad_alloc)
 {
@@ -174,27 +236,10 @@ void nsCore::CRT2400Module::Connect (const nsWireless::CCell& Cell)
 } // Connect()
 
 
-nsWireless::IDriver& nsCore::CRT2400Module::GetDriver () throw()
-{
-    return *m_pDriver;
-
-} // GetDriver()
-
-
-const nsWireless::IDriver& nsCore::CRT2400Module::GetDriver () const throw()
-{
-    return *m_pDriver;
-
-} // GetDriver()
-
-
 void nsCore::CRT2400Module::ApplyProfile (unsigned Pos)
                                                 throw (nsErrors::CSystemExc)
 {
-    m_pDriver->SetChannel (m_ProfilesFile [Pos].GetChannel());
-    m_pDriver->SetMode (m_ProfilesFile [Pos].GetMode());
-    m_pDriver->SetEncryption (m_ProfilesFile [Pos].GetEncryptionD());
-    m_pDriver->SetSSID (m_ProfilesFile [Pos].GetSSID());  
+    ApplyRT2400Profile (m_ProfilesFile [Pos], *m_pDriver);
 
 } // ApplyProfile()
 
@@ -211,29 +256,19 @@ nsUserData::CProfile* nsCore::CRT2400Module::EditNewProfile () throw()
 } // EditNewProfile()
 
 
-nsCore::CRT2400Module::~CRT2400Module () throw (nsErrors::CException)
+void nsCore::CRTAdvancedCLIModule::ApplyProfile (unsigned Pos)
+                                                    throw (nsErrors::CSystemExc)
 {
-    m_ProfilesFile.Record();
+    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);
 
-} // ~CRT2400Module()
+} // ApplyProfile()
 
 
-nsCore::CRT2500Module::~CRT2500Module () throw (nsErrors::CException)
-{
-    m_ProfilesFile.Record();
-
-} // ~CRT2500Module()
-
-
-nsCore::CRT2500Module::CRT2500Module (const std::string& DeviceName,
-                                      const std::string& Path,
-                                      nsGUI::CGtkGUI* pUI)
+nsCore::CRTAdvancedModule::CRTAdvancedModule
+        (nsWireless::CRTAdvancedDriver* pDriver, nsGUI::CConfigDialog* pDialog,
+         const std::string& Path, nsGUI::CGtkGUI* pUI, bool AdHofdmSupport)
                                   throw (nsErrors::CException, std::bad_alloc)
-    : m_pDriver (new nsWireless::CRT2500Driver (DeviceName)),
-      m_ProfileEditor (new nsGUI::CRT25x0ConfigDialog
-                      (m_pDriver->GetSupportedFreq(),
-                       TabWPAPSKRT2500Encrypt, TabWPAPSKRT2500EncryptSize)),
-      m_ProfilesFile (Path),
+    : m_pDriver (pDriver), m_ProfileEditor (pDialog), m_ProfilesFile (Path),
       m_pTable (GTK_TABLE (::gtk_table_new (5, 2, true))), m_pUI (pUI),
       m_CurrentActiveBoxItem (0), m_IsSkipped (false)
 {
@@ -263,34 +298,26 @@ nsCore::CRT2500Module::CRT2500Module (const std::string& DeviceName,
                        &TxBurstCallBack);
     CreateCheckButton ("Turbo rate", 3, 1, m_pDriver->GetTurboRate(),
                                  &TurboRateCallBack);
-    CreateCheckButton ("OFDM in ad-hoc mode", 4, 0,
-                       m_pDriver->GetAdHocOFDM(), &AdHocOFDMCallBack);
+    CreateCheckButton ("Injection in monitor mode", 4, 0,
+                       m_pDriver->GetRfmontx(), &RfmontxCallBack);
+    if (AdHofdmSupport)
+        CreateCheckButton ("OFDM in ad-hoc mode", 4, 1,
+                           m_pDriver->GetAdHocOFDM(), &AdHocOFDMCallBack);
 
-    if (m_pDriver->IsRfmontxSupported())
-        CreateCheckButton ("Injection in monitor mode", 4, 1,
-                           m_pDriver->GetRfmontx(), &RfmontxCallBack);
-
-} // CRT2500Module()
+} // CRTAdvancedModule()
 
 
-nsWireless::IDriver& nsCore::CRT2500Module::GetDriver () throw()
+void nsCore::CRTAdvancedModule::RecordProfiles () throw (nsErrors::CException)
 {
-    return *m_pDriver;
+    m_ProfilesFile.Record();
 
-} // GetDriver()
+} // RecordProfiles()
 
 
-const nsWireless::IDriver& nsCore::CRT2500Module::GetDriver () const throw()
-{
-    return *m_pDriver;
-
-} // GetDriver()
-
-
-void nsCore::CRT2500Module::CreateCBox
+void nsCore::CRTAdvancedModule::CreateCBox
         (const char* TabName [], unsigned TabSize, int CurrentActive,
          const char* Label, unsigned Row,
-         void (*CallBack) (::GtkComboBox*, CRT2500Module*)) throw()
+         void (*CallBack) (::GtkComboBox*, CRTAdvancedModule*)) throw()
 {
 
     ::GtkComboBox* const pCBox (GTK_COMBO_BOX (::gtk_combo_box_new_text()));
@@ -309,9 +336,9 @@ void nsCore::CRT2500Module::CreateCBox
 } // CreateCBox()
 
 
-void nsCore::CRT2500Module::CreateCheckButton (
+void nsCore::CRTAdvancedModule::CreateCheckButton (
          const char* Label, unsigned Row, unsigned Column, bool CurrentState,
-         void (*CallBack) (::GtkToggleButton*, CRT2500Module*)) throw()
+         void (*CallBack) (::GtkToggleButton*, CRTAdvancedModule*)) throw()
 {
     ::GtkWidget* const pButton (::gtk_check_button_new_with_label (Label));
     ::gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pButton), CurrentState);
@@ -323,7 +350,7 @@ void nsCore::CRT2500Module::CreateCheckButton (
 } // CreateCheckButton()
 
 
-nsUserData::CProfile* nsCore::CRT2500Module::EditNewProfile () throw()
+nsUserData::CProfile* nsCore::CRTAdvancedModule::EditNewProfile () throw()
 {
     if (!m_ProfileEditor.Edit (m_ProfilesFile.back()))
     {
@@ -335,122 +362,67 @@ nsUserData::CProfile* nsCore::CRT2500Module::EditNewProfile () throw()
 } // EditNewProfile()
 
 
-void nsCore::CRT2500Module::Connect (const nsWireless::CCell& Cell)
+void nsCore::CRTAdvancedModule::Connect (const nsWireless::CCell& Cell)
                                 throw (nsErrors::CSystemExc, std::bad_alloc)
 {
     m_pDriver->SetMode (nsWireless::GetMatchingMode (Cell.GetMode()));
-    if (Cell.GetMode() == nsWireless::AdHoc);
-        m_pDriver->SetChannel (Cell.GetChannel());
+    m_pDriver->SetChannel (Cell.GetChannel());
     m_pDriver->SetSSID (Cell.GetSSID());
-    m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());
     m_pDriver->SetEncryption (Cell.GetEncryptionD());
+    m_pDriver->SetSSID (Cell.GetSSID());
+    try
+    {
+        m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());
+    }
+    catch (...) {} // It's not useful anyway.
 
 } // Connect()
 
 
-void nsCore::CRT2500Module::ApplyProfile (unsigned Pos)
+void nsCore::CRTAdvancedModule::ApplyProfile (unsigned Pos)
                                                 throw (nsErrors::CSystemExc)
 {
-    if (m_ProfilesFile [Pos].GetMode() == nsWireless::AdHoc)
-        m_pDriver->SetChannel (m_ProfilesFile [Pos].GetChannel());
-    m_pDriver->SetMode (m_ProfilesFile [Pos].GetMode());
-    m_pDriver->SetSSID (m_ProfilesFile [Pos].GetSSID());
-    m_pDriver->SetEncryption (m_ProfilesFile [Pos].GetEncryptionD());
+    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);
 
 } // ApplyProfile()
 
 
-void nsCore::CRT2500Module::SetDriver (nsWireless::CRT2500Driver* pDriver)
-                                                                        throw()
-{
-    m_pDriver.reset (pDriver);
-
-} // SetDriver()
-
-
-void nsCore::CRT2500Module::BecomeRoot () throw (nsErrors::CSystemExc,
-                                                 std::bad_alloc)
-{
-    try
-    {
-        SetDriver (new nsWireless::CRT2500DriverRoot
-                                                (m_pDriver->GetDeviceName()));
-    }
-    catch (const nsErrors::CException& Exc)
-    {
-        throw nsErrors::CSystemExc (Exc.GetMsg(), Exc.GetCode());
-    }
-
-} // BecomeRoot()
-
-
-void nsCore::CRT2500Module::TxBurstCallBack (::GtkToggleButton* pButton,
-                                             CRT2500Module* This) throw()
-{
-    This->CommonCButtonCallBack (&nsWireless::CRT2500Driver::SetTxBurst,
-                                 pButton);
-
-} // TxBurstCallBack()
-
-
-void nsCore::CRT2500Module::TurboRateCallBack (::GtkToggleButton* pButton,
-                                               CRT2500Module* This) throw()
-{
-    This->CommonCButtonCallBack (&nsWireless::CRT2500Driver::SetTurboRate,
-                                 pButton);
-
-} // TurboRateCallBack()
-
-
-void nsCore::CRT2500Module::AdHocOFDMCallBack (::GtkToggleButton* pButton,
-                                               CRT2500Module* This) throw()
-{
-    This->CommonCButtonCallBack (&nsWireless::CRT2500Driver::SetAdHocOFDM,
-                                 pButton);
-
-} // AdHocOFDMCallBack()
-
-
-void nsCore::CRT2500Module::RfmontxCallBack (::GtkToggleButton* pButton,
-                                             CRT2500Module* This) throw()
-{
-    This->CommonCButtonCallBack (&nsWireless::CRT2500Driver::SetRfmontx,
-                                 pButton);
-
-} // RfmontxCallBack()
-
-
-void nsCore::CRT2500Module::ModeCallBack (::GtkComboBox* pCBox,
-                                          CRT2500Module* This) throw()
+void nsCore::CRTAdvancedModule::ModeCallBack (::GtkComboBox* pCBox,
+                                             CRTAdvancedModule* This) throw()
 {
     This->CommonCBoxCallBack (reinterpret_cast<DriverCBoxMember_t>
-                                (&nsWireless::CRT2500Driver::SetWirelessMode),
+                            (&nsWireless::CRTAdvancedDriver::SetWirelessMode),
                               pCBox);
+    This->m_pUI->RefreshRates();
 
 } // ModeCallBack()
 
 
-void nsCore::CRT2500Module::BGProtectionCallBack (::GtkComboBox* pCBox,
-                                                 CRT2500Module* This) throw()
+void nsCore::CRT2500Module::BecomeRoot () throw (nsErrors::CException,
+                                                 std::bad_alloc)
 {
-    This->CommonCBoxCallBack (reinterpret_cast<DriverCBoxMember_t>
-                                (&nsWireless::CRT2500Driver::SetBGProtection),
-                              pCBox);
+    SetDriver (new nsWireless::CRT2500DriverRoot (GetDriver().GetDeviceName()));
 
-} // BGProtectionCallBack()
+} // BecomeRoot()
 
 
-void nsCore::CRT2500Module::TxPreambleCallBack (::GtkComboBox* pCBox,
-                                                CRT2500Module* This) throw()
+void nsCore::CRT61Module::BecomeRoot () throw (nsErrors::CException,
+                                               std::bad_alloc)
 {
-    This->CommonCBoxCallBack (reinterpret_cast<DriverCBoxMember_t>
-                                (&nsWireless::CRT2500Driver::SetTxPreamble),
-                              pCBox);
+    SetDriver (new nsWireless::CRT61DriverRoot (GetDriver().GetDeviceName()));
 
-} // TxPreambleCallBack()
+} // BecomeRoot()
 
 
-void nsCore::CRT2500Module::CommonCBoxCallBack (DriverCBoxMember_t Member,
+void nsCore::CRT73Module::BecomeRoot () throw (nsErrors::CException,
+                                               std::bad_alloc)
+{
+    SetDriver (new nsWireless::CRT73DriverRoot (GetDriver().GetDeviceName()));
+
+} // BecomeRoot()
+
+
+void nsCore::CRTAdvancedModule::CommonCBoxCallBack (DriverCBoxMember_t Member,
                                                 ::GtkComboBox* pCBox) throw()
 {
     try
@@ -467,7 +439,7 @@ void nsCore::CRT2500Module::CommonCBoxCallBack (DriverCBoxMember_t Member,
             else ResumePreviousState (pCBox);
         }
     }
-    catch (const nsErrors::CSystemExc& Exc)
+    catch (const nsErrors::CException& Exc)
     {
         ResumePreviousState (pCBox);
         m_pUI->ProcessError (Exc);
@@ -476,7 +448,7 @@ void nsCore::CRT2500Module::CommonCBoxCallBack (DriverCBoxMember_t Member,
 } // CommonCBoxCallBack()
 
 
-void nsCore::CRT2500Module::CommonCButtonCallBack
+void nsCore::CRTAdvancedModule::CommonCButtonCallBack
             (DriverButtonMember_t Member, ::GtkToggleButton* pButton) throw()
 {
     try
@@ -494,7 +466,7 @@ void nsCore::CRT2500Module::CommonCButtonCallBack
             }
         }
     }
-    catch (const nsErrors::CSystemExc& Exc)
+    catch (const nsErrors::CException& Exc)
     {
         m_pUI->ProcessError (Exc);
     }
@@ -502,12 +474,12 @@ void nsCore::CRT2500Module::CommonCButtonCallBack
 } // CommonCButtonCallBack()
 
 
-void nsCore::CRT2500Module::CBoxFocusCallBack (::GtkComboBox* pCBox, bool,
-                                        nsCore::CRT2500Module* This) throw()
+void nsCore::CRT2570CLIModule::ApplyProfile (unsigned Pos)
+                                                    throw (nsErrors::CSystemExc)
 {
-    This->m_CurrentActiveBoxItem = ::gtk_combo_box_get_active (pCBox);
+    ApplyRTxxProfile (m_ProfilesFile [Pos], m_Driver);
 
-} // CBoxFocusCallBack()
+} // ApplyProfile()
 
 
 nsCore::CRT2570Module::CRT2570Module (const std::string& DeviceName,
@@ -515,58 +487,43 @@ nsCore::CRT2570Module::CRT2570Module (const std::string& DeviceName,
                                       nsGUI::CGtkGUI* pUI)
                                 throw (nsErrors::CException, std::bad_alloc)
     : m_pDriver (new nsWireless::CRT2570Driver (DeviceName)),
-      m_ProfileEditor (new nsGUI::CRT25x0ConfigDialog
-                      (m_pDriver->GetSupportedFreq(),
-                       TabWPAPSKRT2570Encrypt, TabWPAPSKRT2570EncryptSize)),
-      m_ProfilesFile (Path), m_pUI (pUI),
-      m_pBox (GTK_BOX (::gtk_vbox_new (true, 2))), m_CurrentActiveBoxItem (0),
-      m_IsSkipped (false)
+      m_ProfileEditor (new nsGUI::CRTxxConfigDialog()), m_ProfilesFile (Path),
+      m_pUI (pUI), m_pBox (GTK_BOX (::gtk_vbox_new (true, 2))),
+      m_CurrentActiveBoxItem (0), m_IsSkipped (false)
 {
+    m_ProfileEditor.GetDialog()->SetAvailableChannel
+                                                (m_pDriver->GetSupportedFreq());
     m_ProfilesFile.Extract();
     BuildPrivateTab();
 
 } // CRT2570Module()
 
 
-void nsCore::CRT2570Module::BecomeRoot () throw (nsErrors::CSystemExc,
+void nsCore::CRT2570Module::RecordProfiles () throw (nsErrors::CException)
+{
+    m_ProfilesFile.Record();
+
+} // RecordProfiles()
+
+
+void nsCore::CRT2570Module::BecomeRoot () throw (nsErrors::CException,
                                                  std::bad_alloc)
 {
-    try
-    {
-        m_pDriver.reset (new nsWireless::CRT2570DriverRoot
+    m_pDriver.reset (new nsWireless::CRT2570DriverRoot
                                                 (m_pDriver->GetDeviceName()));
-    }
-    catch (const nsErrors::CException& Exc)
-    {
-        throw nsErrors::CSystemExc (Exc.GetMsg(), Exc.GetCode());
-    }
 
 } // BecomeRoot()
 
 
-nsWireless::IDriver& nsCore::CRT2570Module::GetDriver () throw()
-{
-    return *m_pDriver;
-
-} // GetDriver()
-
-
-const nsWireless::IDriver& nsCore::CRT2570Module::GetDriver () const throw()
-{
-    return *m_pDriver;
-
-} // GetDriver()
-
-
 void nsCore::CRT2570Module::Connect (const nsWireless::CCell& Cell)
-                                throw (nsErrors::CSystemExc, std::bad_alloc)
+                                    throw (nsErrors::CSystemExc, std::bad_alloc)
 {
     m_pDriver->SetMode (nsWireless::GetMatchingMode (Cell.GetMode()));
     m_pDriver->SetChannel (Cell.GetChannel());
     m_pDriver->SetSSID (Cell.GetSSID());
-    m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());
     m_pDriver->SetEncryption (Cell.GetEncryptionD());
-    // ? FIXME ? Does the SSID need to be set again? (WPA)
+    m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());
+    m_pDriver->SetSSID (Cell.GetSSID());
 
 } // Connect()
 
@@ -574,23 +531,12 @@ void nsCore::CRT2570Module::Connect (const nsWireless::CCell& Cell)
 void nsCore::CRT2570Module::ApplyProfile (unsigned Pos)
                                                 throw (nsErrors::CSystemExc)
 {
-    if (m_ProfilesFile [Pos].GetMode() == nsWireless::AdHoc)
-        m_pDriver->SetChannel (m_ProfilesFile [Pos].GetChannel());
-    m_pDriver->SetMode (m_ProfilesFile [Pos].GetMode());
-    m_pDriver->SetSSID (m_ProfilesFile [Pos].GetSSID());
-    m_pDriver->SetEncryption (m_ProfilesFile [Pos].GetEncryptionD());
+    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);
     // ? FIXME ? Does the SSID need to be set again? (WPA)
 
 } // ApplyProfile()
 
 
-nsCore::CRT2570Module::~CRT2570Module () throw (nsErrors::CException)
-{
-    m_ProfilesFile.Record();
-
-} // ~CRT2570Module()
-
-
 nsUserData::CProfile* nsCore::CRT2570Module::EditNewProfile () throw()
 {
     if (!m_ProfileEditor.Edit (m_ProfilesFile.back()))
@@ -603,14 +549,6 @@ nsUserData::CProfile* nsCore::CRT2570Module::EditNewProfile () throw()
 } // EditNewProfile()
 
 
-void nsCore::CRT2570Module::CBoxFocusCallBack (::GtkComboBox* pCBox, bool,
-                                        nsCore::CRT2570Module* This) throw()
-{
-    This->m_CurrentActiveBoxItem = ::gtk_combo_box_get_active (pCBox);
-
-} // CBoxFocusCallBack()
-
-
 void nsCore::CRT2570Module::CommonCButtonCallBack
             (DriverButtonMember_t pMember, ::GtkToggleButton* pButton) throw()
 {
@@ -629,7 +567,7 @@ void nsCore::CRT2570Module::CommonCButtonCallBack
             }
         }
     }
-    catch (const nsErrors::CSystemExc& Exc)
+    catch (const nsErrors::CException& Exc)
     {
         m_pUI->ProcessError (Exc);
     }
@@ -637,35 +575,6 @@ void nsCore::CRT2570Module::CommonCButtonCallBack
 } // CommonCButtonCallBack()
 
 
-void nsCore::CRT2570Module::RfmontxCallBack (::GtkToggleButton* pButton,
-                                             CRT2570Module* This) throw()
-{
-    This->CommonCButtonCallBack (&nsWireless::CRT2570Driver::SetRfmontx,
-                                 pButton);
-
-} // RfmontxCallBack()
-
-
-void nsCore::CRT2570Module::PrismHeaderCallBack (::GtkComboBox* pCBox,
-                                                 CRT2570Module* This) throw()
-{
-    This->CommonCBoxCallBack (reinterpret_cast<DriverBoxMember_t>
-                                (&nsWireless::CRT2570Driver::SetPrismHeader),
-                              pCBox);
-
-} // PrismHeaderCallBack()
-
-
-void nsCore::CRT2570Module::AdhocModeCallBack (::GtkComboBox* pCBox,
-                                               CRT2570Module* This) throw()
-{
-    This->CommonCBoxCallBack (reinterpret_cast<DriverBoxMember_t>
-                                (&nsWireless::CRT2570Driver::SetAdHocMode),
-                              pCBox);
-
-} // AdhocModeCallBack()
-
-
 void nsCore::CRT2570Module::CommonCBoxCallBack (DriverBoxMember_t pMember,
                                                 ::GtkComboBox* pCBox) throw()
 {
@@ -682,7 +591,7 @@ void nsCore::CRT2570Module::CommonCBoxCallBack (DriverBoxMember_t pMember,
             else ResumePreviousState (pCBox);
         }
     }
-    catch (const nsErrors::CSystemExc& Exc)
+    catch (const nsErrors::CException& Exc)
     {
         ResumePreviousState (pCBox);
         m_pUI->ProcessError (Exc);
@@ -697,7 +606,8 @@ void nsCore::CRT2570Module::BuildPrivateTab () throw()
     ::gtk_combo_box_append_text (pCBox, "Standard 11Mpbs");
     ::gtk_combo_box_append_text (pCBox, "Auto");
     ::gtk_combo_box_append_text (pCBox, "54Mbps only");
-    ::gtk_combo_box_set_active (pCBox, 0);
+        // Warning : works because of the numeric values, can easily break.
+    ::gtk_combo_box_set_active (pCBox, m_pDriver->GetAdHocMode());
     ::g_signal_connect (G_OBJECT (pCBox), "changed",
                         G_CALLBACK (AdhocModeCallBack), this);
     ::g_signal_connect (G_OBJECT (pCBox), "set-focus-child",
@@ -707,30 +617,64 @@ void nsCore::CRT2570Module::BuildPrivateTab () throw()
     ::gtk_box_pack_start_defaults (pHbox, GTK_WIDGET (pCBox));
     ::gtk_box_pack_start_defaults (m_pBox, GTK_WIDGET (pHbox));
 
-    if (m_pDriver->IsRfmontxSupported())
-    {
-        ::GtkWidget* const pButton (::gtk_toggle_button_new_with_label
+    ::GtkWidget* const pButton (::gtk_toggle_button_new_with_label
                                                 ("Injection in monitor mode"));
-        ::g_signal_connect (G_OBJECT (pButton), "toggled",
-                            G_CALLBACK (RfmontxCallBack), this);
-        ::gtk_box_pack_start_defaults (m_pBox, pButton);
-    }
-    if (m_pDriver->IsPrismHeaderSupported())
-    {
-        pCBox = GTK_COMBO_BOX (::gtk_combo_box_new_text());
-        ::gtk_combo_box_append_text (pCBox, "Default (auto)");
-        ::gtk_combo_box_append_text (pCBox, "Force");
-        ::gtk_combo_box_append_text (pCBox, "Force no");
-        ::gtk_combo_box_set_active (pCBox, 0);
-        ::g_signal_connect (G_OBJECT (pCBox), "changed",
-                            G_CALLBACK (PrismHeaderCallBack), this);
-        ::g_signal_connect (G_OBJECT (pCBox), "set-focus-child",
-                            G_CALLBACK (CBoxFocusCallBack), this);
-        pHbox = GTK_BOX (::gtk_hbox_new (true, 2));
-        ::gtk_box_pack_start_defaults (pHbox, ::gtk_label_new
-                                                            ("Prism header"));
-        ::gtk_box_pack_start_defaults (pHbox, GTK_WIDGET (pCBox));
-        ::gtk_box_pack_start_defaults (m_pBox, GTK_WIDGET (pHbox));
-    }
+    ::gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pButton),
+                                    m_pDriver->GetRfmontx());
+    ::g_signal_connect (G_OBJECT (pButton), "toggled",
+                        G_CALLBACK (RfmontxCallBack), this);
+    ::gtk_box_pack_start_defaults (m_pBox, pButton);
+
+    pCBox = GTK_COMBO_BOX (::gtk_combo_box_new_text());
+    ::gtk_combo_box_append_text (pCBox, "Default (auto)");
+    ::gtk_combo_box_append_text (pCBox, "Force");
+    ::gtk_combo_box_append_text (pCBox, "Force no");
+        // Warning : works because of the numeric values, can easily break.
+    ::gtk_combo_box_set_active (pCBox, m_pDriver->GetPrismHeader());
+    ::g_signal_connect (G_OBJECT (pCBox), "changed",
+                        G_CALLBACK (PrismHeaderCallBack), this);
+    ::g_signal_connect (G_OBJECT (pCBox), "set-focus-child",
+                        G_CALLBACK (CBoxFocusCallBack), this);
+    pHbox = GTK_BOX (::gtk_hbox_new (true, 2));
+    ::gtk_box_pack_start_defaults (pHbox, ::gtk_label_new ("Prism header"));
+    ::gtk_box_pack_start_defaults (pHbox, GTK_WIDGET (pCBox));
+    ::gtk_box_pack_start_defaults (m_pBox, GTK_WIDGET (pHbox));
 
 } // BuildPrivateTab()
+
+
+nsCore::CRT2500Module::CRT2500Module (const std::string& DeviceName,
+                            const std::string& ProfilePath, nsGUI::CGtkGUI* pUI)
+                                    throw (nsErrors::CException, std::bad_alloc)
+    : CRTAdvancedModule (new nsWireless::CRT2500Driver (DeviceName),
+                         new nsGUI::CRT2500ConfigDialog(), ProfilePath, pUI,
+                         true)
+{
+    GetProfileEditor().GetDialog()->SetAvailableChannel
+                                            (GetDriver().GetSupportedFreq());
+
+} // CRT2500Module()
+
+
+nsCore::CRT61Module::CRT61Module (const std::string& DeviceName,
+                            const std::string& ProfilePath, nsGUI::CGtkGUI* pUI)
+                                    throw (nsErrors::CException, std::bad_alloc)
+    : CRTAdvancedModule (new nsWireless::CRT61Driver (DeviceName),
+                         new nsGUI::CRTxxConfigDialog, ProfilePath, pUI, false)
+{
+    GetProfileEditor().GetDialog()->SetAvailableChannel
+                                            (GetDriver().GetSupportedFreq());
+
+} // CRT61Module()
+
+
+nsCore::CRT73Module::CRT73Module (const std::string& DeviceName,
+                            const std::string& ProfilePath, nsGUI::CGtkGUI* pUI)
+                                    throw (nsErrors::CException, std::bad_alloc)
+    : CRTAdvancedModule (new nsWireless::CRT73Driver (DeviceName),
+                         new nsGUI::CRTxxConfigDialog, ProfilePath, pUI, true)
+{
+    GetProfileEditor().GetDialog()->SetAvailableChannel
+                                            (GetDriver().GetSupportedFreq());
+
+} // CRT73Module()
