@@ -25,6 +25,9 @@
 #include <cstring> // strncpy()
 #include <cerrno>
 #include <algorithm> // sort()
+#ifndef NDEBUG
+#include <iostream>
+#endif // NDEBUG
 
 #include "WE17Driver.h"
 #include "ErrorsCode.h"
@@ -103,6 +106,10 @@ nsWireless::CWE17Driver::CWE17Driver (const std::string& Name)
     }
     catch (const nsErrors::CSystemExc& Exc)
     {
+#ifndef NDEBUG
+        std::cerr << "CWE17Driver::CWE17Driver() Can't get interface"
+                     " information : " << Exc.GetCode() << std::endl;
+#endif // NDEBUG
         if (Exc.GetCode() != EOPNOTSUPP) throw;
     }
     if (m_SupportedFreqVec.empty())
@@ -118,11 +125,19 @@ nsWireless::CWE17Driver::CWE17Driver (const std::string& Name)
 std::string nsWireless::CWE17Driver::GetSSID () const
                                                 throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetSSID()" << std::endl;
+#endif // NDEBUG
     char Buffer [IW_ESSID_MAX_SIZE];
     m_Req.u.data.pointer = reinterpret_cast< ::caddr_t> (Buffer);
     m_Req.u.data.length = IW_ESSID_MAX_SIZE;
     m_Req.u.data.flags = 0;
-    Ioctl (SIOCGIWESSID, "Can't get the SSID.");
+    Ioctl (SIOCGIWESSID, "Can't get SSID.");
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetSSID() : "
+              << std::string (reinterpret_cast<const char*>
+                  (m_Req.u.data.pointer), 0, m_Req.u.data.length) << std::endl;
+#endif // NDEBUG
     if (!m_Req.u.data.length) return std::string();
     return std::string (Buffer, 0, m_Req.u.data.length);
 
@@ -133,10 +148,18 @@ std::string nsWireless::CWE17Driver::GetSSID () const
 void nsWireless::CWE17Driver::SetSSID (const std::string& SSID)
                                                 throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::SetSSID() " << SSID << std::endl;
+#endif // NDEBUG
     m_Req.u.data.pointer = const_cast< ::caddr_t> (SSID.c_str());
+#if WIRELESS_EXT < 21
     m_Req.u.data.length = SSID.size() + 1 > IW_ESSID_MAX_SIZE
                                                           ? IW_ESSID_MAX_SIZE  
                              /* + 1 for the '\0'. */      : SSID.size() + 1;
+#else
+    m_Req.u.data.length = SSID.size() > IW_ESSID_MAX_SIZE ? IW_ESSID_MAX_SIZE  
+                                                          : SSID.size();
+#endif // WIRELESS_EXT
     m_Req.u.data.flags = 1; // Only use *this* SSID.
     Ioctl (SIOCSIWESSID, std::string ("Can't set SSID : ") + SSID);
 
@@ -156,6 +179,9 @@ unsigned nsWireless::CWE17Driver::GetChannel () const
 void nsWireless::CWE17Driver::SetChannel (unsigned Channel)
                                                 throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::SetChannel() " << Channel << std::endl;
+#endif // NDEBUG
     m_Req.u.freq.m = Channel;
     m_Req.u.freq.e = 0;
 #if WIRELESS_EXT > 16
@@ -179,6 +205,9 @@ double nsWireless::CWE17Driver::GetFrequency () const
 void nsWireless::CWE17Driver::SetFrequency (double F)
                                                 throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::SetFrequency() " << F << std::endl;
+#endif // NDEBUG
     m_Req.u.freq.m = unsigned (F * 1e8);
     m_Req.u.freq.e = 1;
 #if WIRELESS_EXT > 16
@@ -193,13 +222,16 @@ void nsWireless::CWE17Driver::SetFrequency (double F)
 nsWireless::CEncryptionD nsWireless::CWE17Driver::GetEncryption ()
                                 throw (nsErrors::CSystemExc, std::bad_alloc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetEncryption()" << std::endl;
+#endif // NDEBUG
         /* ? FIXME ? The key index is not considered and only the first key
                      is reported. */
     char Buffer [IW_ENCODING_TOKEN_MAX];
     m_Req.u.data.pointer = reinterpret_cast< ::caddr_t> (Buffer);
     m_Req.u.data.length = IW_ENCODING_TOKEN_MAX;
     m_Req.u.data.flags = 0;
-    Ioctl (SIOCGIWENCODE, "Can't get Wep status.");
+    Ioctl (SIOCGIWENCODE, "Can't get WEP status.");
     return MakeCEncryption (Buffer, m_Req.u.data.length, m_Req.u.data.flags);
 
 } // GetEncryption()
@@ -208,6 +240,14 @@ nsWireless::CEncryptionD nsWireless::CWE17Driver::GetEncryption ()
 void nsWireless::CWE17Driver::SetEncryption (const CEncryptionD& Descriptor)
                                 throw (nsErrors::CSystemExc, std::bad_alloc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::SetEncryption() "
+              << GetAuthName (Descriptor.GetAuth()) << '\t'
+              << GetEncryptName (Descriptor.GetEncrypt());
+    for (unsigned i (0) ; i < CEncryptionD::MaxNbKey ; ++i)
+        std::cerr << "\n\t" << i << " : " << Descriptor.GetKey (i).GetStr();
+    std::cerr << std::endl;
+#endif // NDEBUG
     if (Descriptor.GetEncrypt() == nsWireless::WEP)
     {
         const int AuthFlag (Descriptor.GetAuth() ==
@@ -230,22 +270,28 @@ void nsWireless::CWE17Driver::SetEncryption (const CEncryptionD& Descriptor)
                 m_Req.u.data.length = Descriptor.GetKey (I).Size();
             }
             Ioctl (SIOCSIWENCODE,
-        CatStringNInt (std::string ("Can't set WEP encryption with key : ") +
-                                Descriptor.GetKey (I).GetStr() + " Index : ",
-                       I));
+            CatStringNInt (std::string ("Can't set WEP encryption with key : ")
+                               + Descriptor.GetKey (I).GetStr() + " Index : ",
+                           I));
         }
-        try   // Now activate key 1, it may fail because the previous calls
-        {     // already activate it with some drivers. */
+        try   // Now activate the default key, it may fail because previous
+        {     // calls may have already activated it with some driver.
             m_Req.u.data.pointer = 0;
             m_Req.u.data.length = 0;
-            m_Req.u.data.flags |= IW_ENCODE_NOKEY | 1;
-            Ioctl (SIOCSIWENCODE, "Can't activate key 1.");
+            m_Req.u.data.flags = AuthFlag | IW_ENCODE_NOKEY |
+                                            (Descriptor.GetDefaultKey() + 1);
+            Ioctl (SIOCSIWENCODE, "Can't activate default key.");
+        }
+        catch (const nsErrors::CException& Exc)
+        {
+#ifndef NDEBUG
+            std::cerr << Exc.GetMsg() << ' ' << Exc.GetCode() << std::endl;
+#endif // NDEBUG
         }
-        catch (...) {}
     }
     else
     {
-        m_Req.u.data.flags |= IW_ENCODE_NOKEY | IW_ENCODE_DISABLED;;
+        m_Req.u.data.flags = IW_ENCODE_NOKEY | IW_ENCODE_DISABLED;
         m_Req.u.data.pointer = 0;
         m_Req.u.data.length = 0;
         Ioctl (SIOCSIWENCODE, "Can't disable WEP encryption.");
@@ -257,7 +303,14 @@ void nsWireless::CWE17Driver::SetEncryption (const CEncryptionD& Descriptor)
 nsWireless::Mode_e nsWireless::CWE17Driver::GetMode () const
                                                 throw (nsErrors::CSystemExc)
 {
-    Ioctl (SIOCGIWMODE, "Can't get the mode.");
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetMode()" << std::endl;
+#endif // NDEBUG
+    Ioctl (SIOCGIWMODE, "Can't get operating mode.");
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetMode() " << GetModeName (Mode_e
+                                                (m_Req.u.mode)) << std::endl;
+#endif // NDEBUG
     return Mode_e (m_Req.u.mode);
 
 } // GetMode()
@@ -266,9 +319,12 @@ nsWireless::Mode_e nsWireless::CWE17Driver::GetMode () const
 void nsWireless::CWE17Driver::SetMode (Mode_e Mode)
                                                 throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::SetMode() " << GetModeName (Mode) << std::endl;
+#endif // NDEBUG
     m_Req.u.mode = Mode;
     Ioctl (SIOCSIWMODE,
-           std::string ("Can't set mode : ") + GetModeName (Mode));
+           std::string ("Can't set operating mode : ") + GetModeName (Mode));
 
 } // SetMode()
 
@@ -276,11 +332,14 @@ void nsWireless::CWE17Driver::SetMode (Mode_e Mode)
 nsWireless::CQuality nsWireless::CWE17Driver::GetQuality () const
                                                 throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetQuality()" << std::endl;
+#endif // NDEBUG
     ::iw_statistics Stat;
     m_Req.u.data.pointer = reinterpret_cast < ::caddr_t> (&Stat);
     m_Req.u.data.length = sizeof (Stat);
     m_Req.u.data.flags = 0;
-    Ioctl (SIOCGIWSTATS, "Can't get the signal quality.");
+    Ioctl (SIOCGIWSTATS, "Can't get signal quality.");
     return iw_qualityToCQuality (Stat.qual);
 
 } // GetQuality()
@@ -288,7 +347,14 @@ nsWireless::CQuality nsWireless::CWE17Driver::GetQuality () const
 
 int nsWireless::CWE17Driver::GetTxRate () const throw (nsErrors::CSystemExc)
 {
-    Ioctl (SIOCGIWRATE, "Can't get the Tx rate.");
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetTxRate()" << std::endl;
+#endif // NDEBUG
+    Ioctl (SIOCGIWRATE, "Can't get TX rate.");
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetTxRate() " << m_Req.u.bitrate.value
+              << std::endl;
+#endif // NDEBUG
     return m_Req.u.bitrate.value / TxUnit;
 
 } // GetTxRate()
@@ -297,11 +363,14 @@ int nsWireless::CWE17Driver::GetTxRate () const throw (nsErrors::CSystemExc)
 void nsWireless::CWE17Driver::SetTxRate (int TxRate)
                                                 throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::SetTxRate() " << TxRate << std::endl;
+#endif // NDEBUG
     m_Req.u.bitrate.value = TxRate * TxUnit;
     m_Req.u.bitrate.fixed = 0;  // Hardware should use auto-select.
     m_Req.u.bitrate.disabled = 0;
     m_Req.u.bitrate.flags = 0;
-    Ioctl (SIOCSIWRATE, CatStringNInt ("Can't set Tx rate : ", TxRate));
+    Ioctl (SIOCSIWRATE, CatStringNInt ("Can't set TX rate : ", TxRate));
 
 } // SetTxRate()
 
@@ -309,12 +378,22 @@ void nsWireless::CWE17Driver::SetTxRate (int TxRate)
 nsWireless::CMacAddress nsWireless::CWE17Driver::GetAPMacAddr () const
                                                 throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetAPMacAddr()" << std::endl;
+#endif // NDEBUG
     try {Ioctl (SIOCGIWAP, "Can't get access point Mac address.");}
     catch (const nsErrors::CSystemExc& Exc)
-    {     // I don't consider this as an error.
+    {     // I don't consider this an error.
+#ifndef NDEBUG
+        std::cerr << Exc.GetMsg() << ' ' << Exc.GetCode() << std::endl;
+#endif // NDEBUG
         if (Exc.GetCode() == ENOTCONN) return CMacAddress();
         throw;
     }
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetAPMacAddr() "
+              << std::string (m_Req.u.ap_addr.sa_data, 0, 14) << std::endl;
+#endif // NDEBUG
     return CMacAddress (m_Req.u.ap_addr.sa_data);
 
 } // GetAPMacAddr()
@@ -323,6 +402,9 @@ nsWireless::CMacAddress nsWireless::CWE17Driver::GetAPMacAddr () const
 void nsWireless::CWE17Driver::SetAPMacAddr (const CMacAddress& Addr)
                                 throw (std::bad_alloc, nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::SetAPMacAddr() " << Addr.GetStr() << std::endl;
+#endif // NDEBUG
     strncpy (m_Req.u.ap_addr.sa_data, Addr.Get(), CMacAddress::Size);
     Ioctl (SIOCSIWAP, std::string ("Can't set access point Mac address : ") +
                       Addr.GetStr());
@@ -332,6 +414,9 @@ void nsWireless::CWE17Driver::SetAPMacAddr (const CMacAddress& Addr)
 
 void nsWireless::CWE17Driver::Scan () throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::Scan()" << std::endl;
+#endif // NDEBUG
     m_Req.u.data.pointer = 0;
     m_Req.u.data.length = 0;
         // Full scanning.
@@ -345,21 +430,27 @@ void nsWireless::CWE17Driver::Scan () throw (nsErrors::CSystemExc)
 void nsWireless::CWE17Driver::GetScanResult (std::vector<CCell>& CellVec) const
                                 throw (nsErrors::CException, std::bad_alloc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetScanResult()" << std::endl;
+#endif // NDEBUG
     m_Req.u.data.flags = 0;
     unsigned BufferSize (256); // In iwlist, the starting size is 128.
     unsigned char* Buffer;
-    for ( ; ; ) // *Never ever* a goto :p (especially in C++).
+    for ( ; ; )
     {
         Buffer = new unsigned char [BufferSize];
         m_Req.u.data.pointer = reinterpret_cast < ::caddr_t> (Buffer);
         m_Req.u.data.length = BufferSize;
         try
         {
-            Ioctl (SIOCGIWSCAN, "Can't get the scan results.");
+            Ioctl (SIOCGIWSCAN, "Can't get scan results.");
             break;
         }
         catch (const nsErrors::CSystemExc& Exc)
         {
+#ifndef NDEBUG
+            std::cerr << "\n\tBuffer too small" << std::endl;
+#endif // NDEBUG
             if (Exc.GetCode() == E2BIG)
             {       // If the driver gives the expected size.
                 if (m_Req.u.data.length > BufferSize)
@@ -396,47 +487,62 @@ unsigned nsWireless::CWE17Driver::FillCellVec (const ::iw_event* pEvent,
                                                std::vector<CCell>& Vec) const
                                                 throw (nsErrors::CException)
 {
+#ifndef NDEBUG
+    std::cerr << "\n\tEvent : " << pEvent->cmd;
+#endif // NDEBUG
         // Jean Tourrilhes in iwlib.c warn that data may be unaligned.
     switch (pEvent->cmd)
     {
       case SIOCGIWAP :
+#ifndef NDEBUG
+        std::cerr << "\n\tSIOCGIWAP : "
+                  << std::string (pEvent->u.ap_addr.sa_data, 0, 14);
+#endif // NDEBUG
         Vec.push_back (CCell (CMacAddress (pEvent->u.ap_addr.sa_data)));
       break;
 
       case SIOCGIWMODE :
+#ifndef NDEBUG
+        std::cerr << "\n\tSIOCGIWMODE : "
+                  << GetModeName (Mode_e (pEvent->u.mode));
+#endif // NDEBUG
         Vec.back().m_Mode = Mode_e (pEvent->u.mode);
       break;
 
       case SIOCGIWESSID :
       {
-          /* Fix for new WE version until a CWEXXDriver is implemented.
-           * From wireless.h (v19) : iw_point events are special.
+          /* From wireless.h (v19) : iw_point events are special.
            * First, the payload (extra data) come at the end of the event,
            * so they are bigger than IW_EV_POINT_LEN.
            * Second, we omit the pointer, so start at an offset. */
-#if WIRELESS_EXT > 17
+#if WIRELESS_EXT > 18
         const unsigned CstLength (reinterpret_cast<const ::iw_point*>
         (reinterpret_cast<const char*> (&pEvent->u) - sizeof (void*))->length);
 #else
         const unsigned CstLength (pEvent->u.essid.length);
 #endif
+#ifndef NDEBUG
+        std::cerr << "\n\tSIOCGIWESSID length : " << CstLength << ' ';
+#endif // NDEBUG
         if (CstLength)
             /* From wireless.h (v17) : "Note : in the case of iw_point,
              * the extra data will come at the end of the event". */
             Vec.back().m_SSID = std::string (reinterpret_cast<const char*>
                                                     (pEvent) + IW_EV_POINT_LEN,
                                              0, CstLength);
+#ifndef NDEBUG
+        std::cerr << '#' << Vec.back().GetSSID() << '#';
+#endif // NDEBUG
       }
       break;
 
       case SIOCGIWENCODE :
       {
-          /* Fix for new WE version until a CWEXXDriver is implemented.
-           * From wireless.h (v19) : iw_point events are special.
+          /* From wireless.h (v19) : iw_point events are special.
            * First, the payload (extra data) come at the end of the event,
            * so they are bigger than IW_EV_POINT_LEN.
            * Second, we omit the pointer, so start at an offset. */
-#if WIRELESS_EXT > 17
+#if WIRELESS_EXT > 18
         const unsigned CstLength (reinterpret_cast<const ::iw_point*>
         (reinterpret_cast<const char*> (&pEvent->u) - sizeof (void*))->length);
         unsigned Flags (reinterpret_cast<const ::iw_point*>
@@ -449,49 +555,81 @@ unsigned nsWireless::CWE17Driver::FillCellVec (const ::iw_event* pEvent,
         if (CstLength)
             Data = reinterpret_cast<const char*> (pEvent) + IW_EV_POINT_LEN;
         else Flags |= IW_ENCODE_NOKEY;
+#ifndef NDEBUG
+        std::cerr << "\n\tSIOCGIWENCODE length : " << CstLength << " flags : "
+                  << Flags;;
+#endif // NDEBUG
         Vec.back().m_EncDescriptor = MakeCEncryption (Data, CstLength, Flags);
+#ifndef NDEBUG
+        std::cerr << "\n\t"
+                  << GetAuthName (Vec.back().GetEncryptionD().GetAuth()) << ' '
+                  << GetEncryptName (Vec.back().GetEncryptionD().GetEncrypt());
+#endif // NDEBUG
       }
       break;
 
       case SIOCGIWFREQ :
       {
+#ifndef NDEBUG
+        std::cerr << "\n\tSIOCGIWFREQ : " << pEvent->u.freq.m << ' '
+                  << pEvent->u.freq.e << ' ' << pEvent->u.freq.i << ' '
+                  << pEvent->u.freq.flags;
+#endif // NDEBUG
         if (pEvent->u.freq.m > 1000)
-            try
-            {
-                Vec.back().m_Channel =
-                    GetMatchingFreq (0,
+            Vec.back().m_Channel = GetMatchingFreq (0,
                                 iw_freqToDouble (pEvent->u.freq)).GetChannel();
-            }
-            catch (const nsErrors::CException& Exc)
-            {
-                if (Exc.GetCode() != nsErrors::NoMatchingFreq) throw;
-            }
         else Vec.back().m_Channel = pEvent->u.freq.m;
       }
       break;
 
       case IWEVQUAL :
+#ifndef NDEBUG
+        std::cerr << "\n\tIWEVQUAL";
+#endif // NDEBUG
         Vec.back().m_Quality = iw_qualityToCQuality (pEvent->u.qual);
       break;
 
       case SIOCGIWRATE :
+#ifndef NDEBUG
+        std::cerr << "\n\tSIOCGIWRATE " << pEvent->u.bitrate.value;
+#endif // NDEBUG
         Vec.back().m_Rate =  pEvent->u.bitrate.value / TxUnit;
+      break;
+
+#ifndef NDEBUG
+      default :
+        std::cerr << "Unknown.";
       //break;
+#endif // NDEBUG
     }
+#ifndef NDEBUG
+    std::cerr << "\n\tEvent length : " << pEvent->len << std::endl;
+#endif // NDEBUG
     return pEvent->len;
 
 } // FillCellVec()
 
 
-const nsWireless::CFreq& nsWireless::CWE17Driver::GetMatchingFreq
-        (unsigned Channel, double Frequency) const throw (nsErrors::CException)
+nsWireless::CFreq nsWireless::CWE17Driver::GetMatchingFreq
+                            (unsigned Channel, double Frequency) const throw()
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetMatchingFreq() " << Channel << ' '
+              << Frequency << " : ";
+#endif // NDEBUG
     for (unsigned i (m_SupportedFreqVec.size()) ; i-- ; )
         if (m_SupportedFreqVec [i].GetChannel() == Channel ||
-            m_SupportedFreqVec [i].GetFrequency() == Frequency)
+                            m_SupportedFreqVec [i].GetFrequency() == Frequency)
+        {
+#ifndef NDEBUG
+            std::cerr << "matched.\n";
+#endif // NDEBUG
             return m_SupportedFreqVec [i];
-    throw nsErrors::CException ("Can't find matching Freq.",
-                                nsErrors::NoMatchingFreq);
+        }
+#ifndef NDEBUG
+    std::cerr << "no match.\n";
+#endif // NDEBUG
+    return CFreq (0U, 0.0);
 
 } // GetMatchingFreq()
 
@@ -499,9 +637,17 @@ const nsWireless::CFreq& nsWireless::CWE17Driver::GetMatchingFreq
 double nsWireless::CWE17Driver::GetDriverFreqValue () const
                                                 throw (nsErrors::CSystemExc)
 {
-    Ioctl (SIOCGIWFREQ, "Can't get the frequency/channel.");
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetDriverFreqValue()\n";
+#endif // NDEBUG
+    Ioctl (SIOCGIWFREQ, "Can't get frequency/channel.");
     if (m_Req.u.freq.m > 1000)
         return iw_freqToDouble (m_Req.u.freq);
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetDriverFreqValue " << m_Req.u.freq.m << ' '
+              << m_Req.u.freq.e << ' ' << m_Req.u.freq.i << ' '
+              << m_Req.u.freq.flags << std::endl;
+#endif // NDEBUG
     return -m_Req.u.freq.m;
 
 } // GetDriverFreqValue())
@@ -510,6 +656,9 @@ double nsWireless::CWE17Driver::GetDriverFreqValue () const
 unsigned nsWireless::CWE17Driver::GetPrivateIoctls
         (::iw_priv_args* Tab, unsigned TabSize) throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetPrivateIoctls()\n";
+#endif // NDEBUG
     m_Data.data.pointer = reinterpret_cast< ::caddr_t> (Tab);
     m_Data.data.length = TabSize;
     m_Data.data.flags = 0;
@@ -522,6 +671,9 @@ unsigned nsWireless::CWE17Driver::GetPrivateIoctls
 
 void nsWireless::CWE17Driver::Commit () throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::Commit()\n";
+#endif // NDEBUG
     m_Data.data.pointer = reinterpret_cast< ::caddr_t> (0);
     m_Data.data.length = 0;
     m_Data.data.flags = 0;
@@ -533,6 +685,9 @@ void nsWireless::CWE17Driver::Commit () throw (nsErrors::CSystemExc)
 void nsWireless::CWE17Driver::GetSupportedRates (std::vector<int>& RatesVec)
                                             const throw (nsErrors::CSystemExc)
 {
+#ifndef NDEBUG
+    std::cerr << "CWE17Driver::GetSupportedRates()\n";
+#endif // NDEBUG
     ::iw_range Range;
     m_Req.u.data.pointer = reinterpret_cast< ::caddr_t> (&Range);
     m_Req.u.data.length = sizeof (Range);
